clc
clear all
close all

% Parameters
mr = 3;    % Kg
mw = 20;   % Kg
Ir = 10;   % Kg*m^2
Iw = 0.06; % Kg*m^2
L = 0.4;   % m
R = 0.2;   % m
g = 9.81;  % m/s^2

% Linearized system
% [wheel position, pendulum position, wheel velocity, pendulum velocity]
A = [0,                                                                                            0,                                1, 0;
     0,                                                                                            0,                                0, 1;
     0,                            -mr^2*R*L^2*g/(R^2*mw*Ir+R^2*mw*mr*L^2+R^2*mr*Ir+Iw*Ir+Iw*mr*L^2),                                0, 0;
     0, -(-R^2*mw*mr*g*L-R^2*mr^2*g*L-Iw*mr*g*L)/(R^2*mw*Ir+R^2*mw*mr*L^2+R^2*mr*Ir+Iw*Ir+Iw*mr*L^2),                                0, 0]
B = [0; 0; (Ir+mr*L^2+mr*R*L)/(R^2*mw*Ir+R^2*mw*mr*L^2+R^2*mr*Ir+Iw*Ir+Iw*mr*L^2); -(mr*R*L+R^2*mw+R^2*mr+Iw)/(R^2*mw*Ir+R^2*mw*mr*L^2+R^2*mr*Ir+Iw*Ir+Iw*mr*L^2)]
C = [1, 0, 0, 0;
     0, 1, 0, 0]
D = zeros(2,1)
lin_seagway = ss(A,B,C,D);

pause;

% Checking the stability
eig(A)

pause;

% Checking observability and controllability
obsv(A,C)
rank(obsv(A,C))

pause;

ctrb(A,B)
rank(ctrb(A,B))

pause;

% Transfer functions
tf_lin_seagway = tf(lin_seagway);
G1 = minreal(tf_lin_seagway(2))
G3 = minreal(tf_lin_seagway(1))
G2 = minreal(G3/G1)

pause;

% Design stabilizing controller
figure,rlocus(G1,-G1)

pause;

[Z_G1,P_G1,K_G1] = zpkdata(G1);
R1 = zpk(P_G1{1}(2),P_G1{1}(2)-4,(2+P_G1{1}(1))^2/K_G1);
figure,rlocus(R1*G1)

pause;

L1 = R1*G1;
F1 = minreal(L1/(1+L1))
figure,pzmap(F1)

pause;

% Design outer loop
L2 = F1*G2
figure,bode(L2),grid

pause;

R2 = 0.0014*tf([10 1],[0.2 1])

pause;

L2 = R2*F1*G2;
figure,margin(L2),grid

pause;

figure,step(L2/(1+L2)),grid

% Discretized controllers
Ts = 2*pi/(50*0.5);

R1d = c2d(R1,Ts,'tustin')
R2d = c2d(R2,Ts,'tustin')

